class Sphere 
{
  Vector3d fCenter;
  double fRadius;
  double fIntersection;
  float fColor[] = new float[3];

  Vector3d fLight;
  Ray fRay;
  Diffuse fDiffuse = new Diffuse ();
  Ambient fAmbient = new Ambient ();

  Sphere (Vector3d center, double radius)
  {
    fCenter = new Vector3d (center.fX, center.fY, center.fZ);
    fRadius = radius;
  }

  Sphere (double center[], double radius)
  {
    fCenter = new Vector3d (center[0], center[1], center[2]);
    fRadius = radius;
  }

  public double nearestIntersection (Ray ray)
  {
    double s, s1, s2, dot, discriminant;
    Vector3d deltaP = new Vector3d ();
    
    fRay = ray;

    deltaP.fX = fCenter.fX - ray.fPosition.fX;
    deltaP.fY = fCenter.fY - ray.fPosition.fY;
    deltaP.fZ = fCenter.fZ - ray.fPosition.fZ;

    dot = ray.fDirection.dotProduct (deltaP); 
    discriminant = (dot * dot) - deltaP.dotProduct (deltaP) + (fRadius * fRadius);

    if (discriminant < 0.0)
    {
      //No intersection
      return -1.0;
    }
    else if (discriminant == 0.0)
    {
      fIntersection = dot;
      return (dot);
    }
    else
    {
      //Find smallest positive result
      discriminant = Math.sqrt (discriminant);

      s1 = dot + discriminant;
      s2 = dot - discriminant;

      if ( s1 < 0)
      {
        if (s2 < 0)
          return -1.0;  //Both are negative

        s = s2;         //s2 is the only positive value
      }
      else
      {
        if (s2 < 0)
          s = s1;       //s1 is the only positive value
        else
          s = (s1 < s2) ? s1 : s2;
      }
    }

    fIntersection = s;
    return (s);
  } //Intersection

  public Vector3d computeNormal (Vector3d intersect)
  {
    Vector3d result = new Vector3d ();

    result.fX = (intersect.fX - fCenter.fX)/fRadius;
    result.fY = (intersect.fY - fCenter.fY)/fRadius;
    result.fZ = (intersect.fZ - fCenter.fZ)/fRadius;

    return result;
  } //computeNormal

  public void computeColor ()
  {
    Vector3d intersect = fRay.ExtrapolatePoint (fIntersection);
    Vector3d normal = computeNormal (intersect);

    fDiffuse.computeIntensity (intersect, normal);
    fAmbient.computeIntensity ();

    fColor[0] = fAmbient.getRed () + fDiffuse.getRed ();
    fColor[1] = fAmbient.getGreen () + fDiffuse.getGreen ();
    fColor[2] = fAmbient.getBlue () + fDiffuse.getBlue ();
  }

  public float getRed ()
  {
    return fColor[0];
  }

  public float getGreen ()
  {
    return fColor[1];
  }

  public float getBlue ()
  {
    return fColor[2];
  }

  public Ambient getAmbient ()
  {
    return fAmbient;
  }

  public Diffuse getDiffuse ()
  {
    return fDiffuse;
  }
} //Sphere